@@ -30,10 +30,22 @@ $JAVA_HOME="";
 #
 $JAVA_EXE="";
 #
-# Additional args to pass to java.
+# Additional args to pass to java before command-line args.
 #
 $JAVA_ARGS="";
 #
+# Additional args to pass to java after command-line args.
+#
+$EXTRA_JAVA_ARGS="-Djava.util.logging.manager=com.caucho.log.LogManagerImpl";
+$EXTRA_JAVA_ARGS.=" -Djavax.management.builder.initial=com.caucho.jmx.MBeanServerBuilderImpl";
+
+#
+# Default stack size.  The 1m is a good tradeoff between stack size and
+# allowing more threads.  The default stack size doesn't allow enough
+# threads for several operating systems.
+#
+$DEFAULT_STACK_SIZE="-Xss1m";
+#
 # Additional args to pass to Resin
 #
 $RESIN_ARGS="";
@@ -77,7 +89,8 @@ sub usage {
     print "  -help           : this usage message\n";
     print "  -verbose        : echo variables and argument before execution\n";
     print "  -java_home dir  : set JAVA_HOME\n";
-    print "  -resin_home dir : set RESIN_HOME\n";
+    print "  -resin-home dir : set RESIN_HOME\n";
+    print "  -server-root dir : set SERVER_ROOT\n";
     print "  -conf <resin.conf> : changes the configuration file\n";
     print "  -classpath cp   : set CLASSPATH\n";
     print "  -native         : force native threads\n";
@@ -85,6 +98,7 @@ sub usage {
     print "  -nojit          : no jit\n";
     print "  -stdout <file>  : stdout log\n";
     print "  -stderr <file>  : stderr log\n";
+    print "  -jvm-log <file> : jvm log\n";
     print "  -pid <file>     : file for the pid\n";
     print "  -no-auto-restart : disable automatic server restart\n";
     print "cmd:\n";
@@ -105,7 +119,9 @@ $keepalive=1;
 $depth="";
 $stdout_log="";
 $stderr_log="";
+$jvm_log="";
 $cmd = "exec";
+$|=1;
 
 while ( $#ARGV >= 0) {
     if ($ARGV[0] eq "-v" || $ARGV[0] eq "-verbose") {
@@ -180,6 +196,11 @@ while ( $#ARGV >= 0) {
 	shift(@ARGV);
 	shift(@ARGV);
     }
+    elsif ($ARGV[0] eq "-jvm-log") {
+	$jvm_log=&normalize_path($ARGV[1]);
+	shift(@ARGV);
+	shift(@ARGV);
+    }
     elsif ($ARGV[0] eq "-pid") {
 	$pid_file = &normalize_path($ARGV[1]);
 	shift(@ARGV);
@@ -191,7 +212,13 @@ while ( $#ARGV >= 0) {
 	shift(@ARGV);
     }
     elsif ($ARGV[0] =~ "^-J") {
-	$JAVA_ARGS .= " " . substr($ARGV[0], 2);
+        $val = substr($ARGV[0], 2);
+	$JAVA_ARGS .= " " . $val;
+
+        if ($val =~ "^-Xss") {
+          $DEFAULT_STACK_SIZE = "";
+        }
+
 	shift(@ARGV);
     }
     elsif ($ARGV[0] =~ "^-D") {
@@ -199,7 +226,13 @@ while ( $#ARGV >= 0) {
 	shift(@ARGV);
     }
     elsif ($ARGV[0] =~ "^-X") {
-	$JAVA_ARGS .= " " . $ARGV[0];
+        $val = $ARGV[0];
+	$JAVA_ARGS .= " " . $val;
+
+        if ($val =~ "^-Xss") {
+          $DEFAULT_STACK_SIZE = "";
+        }
+
 	shift(@ARGV);
     }
     elsif ($ARGV[0] eq "-h" || $ARGV[0] eq "-help" || $ARGV[0] eq "help") {
@@ -265,6 +298,13 @@ while ( $#ARGV >= 0) {
     }
 }
 
+#
+# Add the default values.
+#
+if ($DEFAULT_STACK_SIZE) {
+  $JAVA_ARGS .= " " . $DEFAULT_STACK_SIZE;
+}
+
 # 
 # Find the resin libraries
 #
@@ -400,57 +440,41 @@ if (! $CLASSPATH) {
 if (! $CLASSPATH) {
     $CLASSPATH=".";
 }
+
 if (-d "$RESIN_HOME/classes") {
     $CLASSPATH="$CLASSPATH:$RESIN_HOME/classes";
 }
 
+#
+# These jars match the resin.manifest.  They're the minimal set that
+# Resin needs to start.  Other jars in lib are loaded dynamically
+# with a separate class loader.
+#
+
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jsdk-24.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jta-101.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jca-15.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jstl-11.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/isorelax.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jmx-12.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/portlet-10.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/resin.jar";
+#$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/license.jar";
+
 opendir(RESINLIB, "$RESIN_HOME/lib");
 while ($file = readdir(RESINLIB)) {
-    if ($file !~ /\.jar$/ && $file !~ /\.zip$/) {
-    }
-    elsif ($file =~ /jsdk.*\.jar/) {
-    }
-    elsif ($file eq "jdk12.jar") {
-    }
-    else {
+    if ($file =~ /\.jar$/ || $file =~ /\.zip$/) {
 	$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/$file";
     }
 }
 closedir(RESINLIB);
 
-if ($VMTYPE ne "kaffe") {
-    if (-e "$JAVA_HOME/lib/classes.zip") {
-	$CLASSPATH="$CLASSPATH:$JAVA_HOME/lib/classes.zip";
-    }
-    if (-e "$JAVA_HOME/lib/tools.jar") {
-	$CLASSPATH="$CLASSPATH:$JAVA_HOME/lib/tools.jar";
-    }
-    if (-e "$JAVA_HOME/jre/lib/rt.jar") {
-	$CLASSPATH="$CLASSPATH:$JAVA_HOME/jre/lib/rt.jar";
-    }
-}
-$CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jsdk23.jar";
-if (-e "$JAVA_HOME/lib/rt.jar") {
-  $CLASSPATH="$CLASSPATH:$JAVA_HOME/lib/rt.jar";
-}
-if (-e "$JAVA_HOME/lib/i18n.jar") {
-  $CLASSPATH="$CLASSPATH:$JAVA_HOME/lib/i18n.jar";
-}
+#
+# Add the tools.jar
+#
 
-if ($VMTYPE eq kaffe) {
-    if (-d "/usr/local/share/kaffe") {
-	$CLASSPATH="$CLASSPATH:/usr/local/share/kaffe/Klasses.jar";
-	$CLASSPATH="$CLASSPATH:/usr/local/share/kaffe/pizza.jar";
-    }
-    elsif (-d "/usr/share/kaffe") {
-	$CLASSPATH="$CLASSPATH:/usr/share/kaffe/Klasses.jar";
-	$CLASSPATH="$CLASSPATH:/usr/share/kaffe/pizza.jar";
-    }
-}
-if (-e "$JAVA_HOME/lib/classes.zip" ||
-    -e "$JAVA_HOME/lib/rt.jar" && ! -e "$JAVA_HOME/bin/java" ||
-    $VMTYPE eq "kaffe") {
-  $CLASSPATH="$CLASSPATH:$RESIN_HOME/lib/jdk12.jar";
+if (-e "$JAVA_HOME/lib/tools.jar") {
+  $CLASSPATH="$CLASSPATH:$JAVA_HOME/lib/tools.jar";
 }
 
 #
@@ -460,11 +484,6 @@ if (! $depth) {
     $depth = 4;
 }
 if ($JAVA_EXE) {
-}
-elsif ($VMTYPE eq 'kaffe') {
-  $JAVA_EXE="kaffe -ss 256k";
-} elsif ($VMTYPE eq "japhar") {
-    $JAVA_EXE="/usr/local/japhar/bin/japhar";
 } elsif ($VMTYPE eq "prof") {
     $JAVA_EXE="$JAVA_HOME/bin/java_g";
     $JAVA_ARGS .= " -prof $thread";
@@ -509,6 +528,8 @@ sub handler {
     if ($child > 0) {
 	$SIG{CHLD} = 'IGNORE';
     }
+
+    print "\n\nResin wrapper.pl closing.\n\n";
     
     close(S);
     close(C); # closing the accepted socket should start orderly shutdown
@@ -530,23 +551,29 @@ sub handler {
     exit(1);
 }
 
-$JAVA_ARGS .= " -Dresin.home=$SERVER_ROOT";
+$JAVA_ARGS .= " -Dresin.home=$SERVER_ROOT $EXTRA_JAVA_ARGS";
 
 if ($cmd eq "start" || $cmd eq "restart") {
   mkdir("$SERVER_ROOT/log", 0755);
+
   if (! $stdout_log) {
       $stdout_log = "$SERVER_ROOT/log/stdout.log";
   }
+
   if (! $stderr_log) {
       $stderr_log = "$SERVER_ROOT/log/stderr.log";
   }
+
+  if (! $jvm_log) {
+      $jvm_log = "$SERVER_ROOT/log/jvm.log";
+  }
 }
 
 if ($stdout_log) {
+    $RESIN_ARGS .= " -stdout $stdout_log";
+
     open(TOUCH, ">> $stdout_log") || die "Can't create $stdout_log.\n";
     close(TOUCH);
-    
-    $RESIN_ARGS .= " -stdout $stdout_log";
 }
 
 if ($stderr_log) {
@@ -556,6 +583,11 @@ if ($stderr_log) {
     close(TOUCH);
 }
 
+if ($jvm_log) {
+    open(TOUCH, ">> $jvm_log") || die "Can't create $jvm_log.\n";
+    close(TOUCH);
+}
+
 if ($verbose) {
     print "JAVA_HOME:\t$JAVA_HOME\n";
     print "RESIN_HOME:\t$RESIN_HOME\n";
@@ -619,11 +651,9 @@ if ($cmd eq "start" || $cmd eq "restart") {
       exit(0);
   }
 
-  # set new process group
   setpgrp;
   $SIG{"HUP"} = 'IGNORE';
 
-  # become daemon
   if (fork()) {
       exit(0);
   }
@@ -637,7 +667,7 @@ if ($cmd eq "start" || $cmd eq "restart") {
   close(STDIN);
   # close(STDOUT);
   # close(STDERR);
-  
+
   $SIG{"INT"} = 'handler';
   $SIG{"QUIT"} = 'handler';
   $SIG{"KILL"} = 'handler';
@@ -651,7 +681,7 @@ if ($cmd eq "start" || $cmd eq "restart") {
       $date = `date`;
       chop($date);
 
-      print("Resin $name $cmd at $date\n");
+      print "Resin $name $cmd at $date\n";
 
       # create a keepalive socket
       # when the wrapper dies, the httpd class will detect that and
@@ -662,32 +692,41 @@ if ($cmd eq "start" || $cmd eq "restart") {
       bind(S, $addr) || die "bind: $!";
       $myaddr = getsockname(S);
       ($fam, $port, $addr) = unpack("S n a4 x8", $myaddr);
-      listen(S, 10) || die "connect: $!";
+      listen(S, 5) || die "connect: $!";
 
       if (($child = fork()) == 0) {
+	  close(S);
           close(STDOUT);
           close(STDERR);
-	  close(S);
-          open(STDOUT, ">>$stdout_log");
-          open(STDERR, ">>$stderr_log");
+
+          open(STDOUT, ">>$jvm_log");
+          open(STDERR, ">&STDOUT");
+
 	  exec("$JAVA_EXE $JAVA_ARGS $class -socketwait $port $conf $RESIN_ARGS");
           print(STDERR "Can't start java: $JAVA_EXE $JAVA_ARGS $class $conf $RESIN_ARGS");
 	  exit(66);
       }
       elsif ($child < 0) {
-	  print("Resin $name fork failed at $date\n");
+	  print "Resin $name fork failed at $date\n";
       }
       else {
-	  $rin = '';
-	  $win = '';
-	  vec($rin, fileno(S), 1) = 1;
-	  $timeout = $accept_time;
+          $hasAccept = 0;
 
-	  if (select($rin, $win, $rin, $timeout) > 0) {
+          for ($i = 0; ! $hasAccept && $i < 5; $i++) {
+  	    $rin = '';
+	    $win = '';
+	    vec($rin, fileno(S), 1) = 1;
+	    $timeout = $accept_time / 5;
+
+	    if (select($rin, $win, $rin, $timeout) > 0) {
 	      accept(C, S);
+              $hasAccept = 1;
+
               close(STDOUT);
               close(STDERR);
-	  }
+	    }
+          }
+
 	  close(S);
       }
       
